from django.db.models import Count, Min, Max, Q
from django.http import Http404

from rest_framework import generics, filters, response, views

from .serializers import (
    BrandSerializer, CarModelSerializer,
    CarSerializer, BrandModelListSerializer, ModelPhotoSerializer,

    BrandResultsSerializer,
    CarModelResultsSerializer,
    CarResultsSerializer,
)
from .filters import BrandFilter, CarModelFilter, CarFilter

from publieditor.content.models import (
    Brand, CarModel, Car,
    PhotoCarModelGallery
)


# Plain list views

class BrandListView(generics.ListAPIView):

    queryset = Brand.objects.filter(
        status='active'
    ).prefetch_related(
        'models'
    ).prefetch_related(
        'corporates'
    ).prefetch_related(
        'models__cars'
    )
    serializer_class = BrandSerializer
    filter_class = BrandFilter


class CarModelListView(generics.ListAPIView):

    queryset = CarModel.objects.filter(
        Q(status='online') | Q(status='runout')
    ).prefetch_related('cars').select_related('brand')
    serializer_class = CarModelSerializer
    filter_class = CarModelFilter


class CarListView(generics.ListAPIView):
    queryset = Car.objects.filter(status='ok', model__status__in=('online', 'runout'))
    filter_backends = (filters.DjangoFilterBackend, filters.OrderingFilter,)
    # paginate_by = 100
    # ordering_fields = ('brand', 'model',)  # To be able to use the ordering=brand,-model parameter (see docs).
    ordering = ('brand', 'model')
    filter_class = CarFilter
    serializer_class = CarSerializer


class BrandModelListView(generics.ListAPIView):
    model = CarModel,
    serializer_class = BrandModelListSerializer

    def get_queryset(self):
        date_last_modify_begin = "%s-%s-%s 00:00:00" % (
            self.kwargs['year'], self.kwargs['month'], self.kwargs['day']
        )
        date_last_modify_end = "%s-%s-%s 23:59:59" % (
            self.kwargs['year'], self.kwargs['month'], self.kwargs['day']
        )
        # TODO: da moficare con date_last_modify
        return CarModel.objects.filter(
            mod_date__gte=date_last_modify_begin,
            mod_date__lte=date_last_modify_end,
            brand__isnull=False,
            status__in=('online', 'runout')
        ).order_by('-id').select_related('brand')


class ModelPhotoDetailView(generics.RetrieveAPIView):
    def get_queryset(self):
        return CarModel.objects.filter(
            Q(status='online') | Q(status='runout')
        )

    model = CarModel
    serializer_class = ModelPhotoSerializer


# Results views
class CarCountView(CarListView):

    def get(self, request, *args, **kwargs):
        count = self.filter_queryset(self.get_queryset()).count()
        return response.Response(count)


class ResultView(CarListView):

    def list(self, request, *args, **kwargs):
        queryset = self.get_queryset()

        if 'driving' in request.QUERY_PARAMS and bool(int(request.QUERY_PARAMS['driving'])):
            queryset = queryset.filter(driving=True)
        if 'area_c' in request.QUERY_PARAMS and bool(int(request.QUERY_PARAMS['area_c'])):
            queryset = queryset.filter(area_c=True)

        object_list = self.filter_queryset(queryset)
        count = object_list.count()
        models = object_list\
            .order_by('model')\
            .values('model').annotate(
                minPrice=Min('price'), maxPrice=Max('price'),
                numberOfVersionsForThisResult=Count('model')
            )
        result = {
            'count': count,
            'data': models
        }
        return response.Response(result)


class CarConfigView(CarListView):

    def get(self, request, *args, **kwargs):
        finalObject = {}
        config_object = self.queryset.aggregate(
            maxPrice=       Max('price'),
            minPrice=       Min('price'),
            minYear=        Min('year'),
            maxYear=        Max('year'),
            minLuggage=     Min('luggage'),
            maxLuggage=     Max('luggage'),
            minTank=        Min('tank'),
            maxTank=        Max('tank'),
            minWeight=      Min('weight'),
            maxWeight=      Max('weight'),
            minLength=      Min('length'),
            maxLength=      Max('length'),
            minWidth=       Min('width'),
            maxWidth=       Max('width'),
            minHeight=      Min('height'),
            maxHeight=      Max('height'),
            minDispl=       Min('displ'),
            maxDispl=       Max('displ'),
            minPowerCv=     Min('power_cv'),
            maxPowerCv=     Max('power_cv'),
            minTorqueMax=   Min('torque_max'),
            maxTorqueMax=   Max('torque_max'),
            minSpeed=       Min('speed'),
            maxSpeed=       Max('speed'),
            minAcc=         Min('acc'),
            maxAcc=         Max('acc'),
            minCo2=         Min('co2'),
            maxCo2=         Max('co2'),
            minConsMix=     Min('cons_mix'),
            maxConsMix=     Max('cons_mix'),
            minKmlMix=      Min('kml_mix'),
            maxKmlMix=      Max('kml_mix'),
            minEco=         Min('eco'),
            maxEco=         Max('eco'),
        )
        finalObject['minsAndMaxs'] = config_object
        return response.Response(finalObject)